from collections import Counter

from PyPDF2 import PdfReader


def get_pdf_pagenumber(pdfpath):
    reader = PdfReader(pdfpath)
    return len(reader.pages)


def get_page_size(pdf_page):
    """get pdf pages size in millimeters"""
    """https://stackoverflow.com/questions/46232984/how-to-get-pdf-file-metadata-page-size-using-python"""
    width = float(pdf_page.mediabox.width) * 25.4 / 72
    height = float(pdf_page.mediabox.height) * 25.4 / 72
    return width, height


def page_is_landscape(pdf_page):
    """return if the pdf page is landscape"""
    """
    参考：https://stackoverflow.com/questions/37424416/how-to-get-pdf-orientation-using-pypdf2
    媒体框（media box）定义将用于印刷页面的物理介质的边界。
    旋转属性会覆盖mediaBox的设置，因此需要结合旋转属性进行判断：
    当 宽 > 高，并且不旋转或旋转180度的情况下，判断页面为横向Landscpae，否则为正常竖向
    当 宽 < 高，并且不旋转或旋转180度的情况下，页面为正常竖向，否则则是经过旋转，最终页面呈现效果是横向
    """
    deg = pdf_page.get("/Rotate")
    mediabox = pdf_page.mediabox
    width, height = mediabox.upper_right
    if width > height:
        if deg in [0, 180, None]:
            return True
        else:
            return False
    else:
        if deg in [0, 180, None]:
            return False
        else:
            return True


def get_pdf_info(pdfpath):
    filename = pdfpath.name
    reader = PdfReader(pdfpath)
    page_number = len(reader.pages)
    landscape = [page_is_landscape(page) for page in reader.pages]
    count_landscape = Counter(landscape)
    landscape_pages_num = count_landscape[True]
    page_break_list = page_break(landscape)
    page_landscape = {}
    for i in page_break_list:
        _num = int(i.split('-')[0]) - 1
        page_landscape[i] = landscape[_num]
    return {
        'filename': filename,
        'page_number': page_number,
        'landscape_pages_num': landscape_pages_num,
        'page_break_list': page_break_list,
        'page_landscape': page_landscape,
        'landscape': landscape
    }


def page_break(page_list):
    page_break_list = ['1']
    mark_all = []
    start = 1
    mark = [start]
    number_of_page_list = len(page_list)
    if number_of_page_list == 2:
        return ['1', '2']
    for i in range(1, number_of_page_list):
        if start == number_of_page_list - 2:
            # print("特殊处理{}".format(start))
            if page_list[start] == page_list[start + 1]:
                start += 1
                mark.append(start)
                mark_all.append(mark)
                break
            else:
                end = start + 1
                mark.append(end - 1)
                mark_all.append(mark)
                start = end
                mark = [start, end]
                mark_all.append(mark)
                break
        else:
            if page_list[start] == page_list[start + 1]:
                start += 1
            else:
                end = start + 1
                mark.append(end - 1)
                mark_all.append(mark)
                start = end
                mark = [start]
    # print(mark_all)
    for p in mark_all:
        if p[0] == p[1]:
            page_break_list.append('{}'.format(p[0] + 1))
        else:
            page_break_list.append('{}-{}'.format(p[0] + 1, p[1] + 1))
    return page_break_list



import json
import subprocess
from pathlib import Path

import click

from .pdfinfo import get_pdf_info

cwd = Path.cwd()
packagejson_path = Path.joinpath(cwd, 'package.json')
pyfile_dict = Path(__file__).parent
template_file_path = Path.joinpath(pyfile_dict, 'template.tex')


def get_pdf_files():
    p = Path(cwd)
    return list(p.glob('[1234567890]*.pdf'))


def clean_file():
    aux_file = list(cwd.glob('*.aux'))
    log_file = list(cwd.glob('*.log'))
    out_file = list(cwd.glob('*.out'))
    toc_file = list(cwd.glob('*.toc'))
    clean_list = aux_file + log_file + out_file + toc_file
    for p in clean_list:
        p.unlink()


def runcmd(command):
    ret = subprocess.run(command, shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    if ret.returncode == 0:
        resa = ret.stdout
    else:
        resa = ret.stderr
    return resa


def clean_file_use_latexmk(file_name):
    cmd = ['latexmk', '-c', file_name]
    res = runcmd(cmd)
    return res


def tex_add_pages(filename, pages: str, frame: bool = True, scale: float = 0.75, landscape: bool = False):
    return fr'\includepdf[pages={{{pages}}}, frame={frame}, scale={scale},landscape={landscape}]{{{filename}}}'


def tex_add_firstpage(filename, pages: str, frame: bool = True, scale: float = 0.75, landscape: bool = False):
    _a = filename.split(' ')
    _b = ' '.join(_a[1:])
    section_name = _b.replace(' ', r' \ ')[:-4]
    return fr'\includepdf[pages={{{pages}}}, frame={frame}, scale={scale},landscape={landscape},pagecommand=\section{{{section_name}}}]{{{filename}}} '


def tex_add_pdf(filepath):
    pdfinfo = get_pdf_info(filepath)
    filename = pdfinfo['filename']
    latex_cmd = f'% add {filename}\n'
    latex_cmd += r'\includepdfset{pagecommand={\thispagestyle{fancy}}}'
    page_break_list = pdfinfo['page_break_list']
    page_landscape = pdfinfo['page_landscape']
    for i in page_break_list:
        if i == '1':
            latex_cmd += '\n' + tex_add_firstpage(filename, '1', landscape=page_landscape[i])
        else:
            latex_cmd += '\n' + tex_add_pages(filename, pages=i, landscape=page_landscape[i])
    return latex_cmd


def total_tex_cmd():
    pdfs = get_pdf_files()
    if len(pdfs) == 0:
        raise Exception("当前目录下无PDF文件")
    latexcmd = ''
    for pdf in pdfs:
        latexcmd += tex_add_pdf(pdf) + '\n\n'
    return latexcmd


def generate_tex_file(papername):
    with open(template_file_path, 'r', encoding='utf-8') as f:
        tpl = f.read()
        a = tpl.replace('AAANAMEAAA', papername)
        content = total_tex_cmd()
        b = a.replace('AAACONTENTAAA', content)
        new_file_name = Path.joinpath(cwd, "{}.tex".format(papername))
        with open(new_file_name, 'w', encoding='utf-8') as newfile:
            newfile.write(b)


def init_project():
    project_info = {
        'papername': '',
        'files': [x.name for x in get_pdf_files()]
    }
    with open(packagejson_path, 'w', encoding='utf-8') as f:
        json.dump(project_info, f, indent=4, ensure_ascii=False)
    print(project_info)
    print("需要修改package.json中报告名称papername")


def get_project_info():
    if packagejson_path.exists():
        with open(packagejson_path, 'r', encoding='utf-8') as f:
            j = json.load(f)
            return j
    else:
        print("package.json not exist,creating an empty")
        init_project()
        return {'papername': '', 'files': []}


def save_project_info(project_info):
    with open(packagejson_path, 'w', encoding='utf-8') as f:
        json.dump(project_info, f, indent=4, ensure_ascii=False)


def show_info():
    click.echo(click.style('Project Info:', fg='yellow'))
    click.echo(f"当前目录: {cwd}")
    click.echo(f"tex文件目录: {template_file_path}")
    project_info = get_project_info()
    click.echo(f"项目信息: ")
    click.echo("  papername: {}".format(project_info['papername']))
    click.echo("  files: {}".format(project_info['files']))


def buildpdf(papername):
    print(f"项目名称: {papername}")
    file_name = f"{papername}.tex"
    print(f">>> 生成tex文件: {file_name}", end=' ')
    generate_tex_file(papername)
    print("✓")
    print(">>> 使用xelatex编译", end=' ')
    # cmd = ['xelatex', file_name]
    cmd = ['latexmk', '-interaction=nonstopmode', '-file-line-error', '-pdf', '-xelatex', file_name]
    runcmd(cmd)  # 需要执行两次，生成目录 尝试用latexmk生成
    res = runcmd(cmd)
    print("✓")
    msg = res.split(b'\r\n')[-2]
    try:
        print(msg.decode('gbk'))
    except UnicodeDecodeError:
        print(msg)
    print(">>> 清理文件")
    rclean = clean_file_use_latexmk(file_name)
    msg = rclean.split(b'\r\n')[-2]
    try:
        print(msg.decode('gbk'))
    except UnicodeDecodeError:
        print(msg)
    print(">>> 完成")


@click.group()
def cli():
    pass


@cli.command()
def init():
    """Init Project"""
    click.echo(click.style(f"Init Project", fg='yellow'))
    init_project()


@cli.command()
def info():
    """Show project info"""
    show_info()


@cli.command()
@click.option('-y', 'y', is_flag=True)
def build(y):
    """Build pdf file"""
    project_info = get_project_info()
    show_info()
    if project_info['papername'] == '':
        print("报告名称为空")
        project_info['papername'] = input("请输入报告名称:")
    save_project_info(project_info)
    if not y:
        _y = input("是否现在编译PDF? (y/N) ")
        if _y == 'y':
            buildpdf(project_info['papername'])
        else:
            print("取消")
    else:
        buildpdf(project_info['papername'])


if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mgpdf']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'pypdf2>=3.0.0,<4.0.0']

entry_points = \
{'console_scripts': ['mgpdf = mgpdf.mgpdf:cli']}

setup_kwargs = {
    'name': 'mgpdf',
    'version': '0.0.2',
    'description': '',
    'long_description': '# mgpdf\n\n[![PyPI - Version](https://img.shields.io/pypi/v/mgpdf.svg)](https://pypi.org/project/mgpdf)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/mgpdf.svg)](https://pypi.org/project/mgpdf)\n\n-----\n\n**Table of Contents**\n\n- [Installation](#installation)\n- [License](#license)\n\n## Installation\n\n```console\npip install mgpdf\n```\n\n## License\n\n`mgpdf` is distributed under the terms of the [MIT](https://spdx.org/licenses/MIT.html) license.\n',
    'author': 'liuxsdev',
    'author_email': '1028330144@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/liuxsdev/mgpdf',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
